/*
 * $Id: DccmDevice.cs 86 2007-03-28 13:19:41Z lloydd $
 */
 
using System;
using NDesk.DBus;

namespace org.wimo.dccm
{
   [Interface ("org.wimo.dccm.Device")]
   public interface IDevice
   {
      string Path {get;}
      uint OsVersionMajor {get;}
      uint OsVersionMinor {get;}
      uint Version {get;}
      string DeviceIP {get;}
      string DeviceRapiIP {get;}
      uint DeviceRapiPort {get;}
      uint CpuType {get;}
      uint CurrentPartnerId {get;}
      uint Id {get;}
      string Name {get;}
      string PlatformName {get;}
      string ModelName {get;}
      string Guid {get;}
      bool PasswordProtected {get;set;}
      bool PasswordProvided {get;set;}
      string Password {get;set;}
      uint Key {get;set;}
   }
   
   public class Device : IDevice
   {
      string path;
      uint os_major;
      uint os_minor;
      uint version;
      uint cpu_type;
      uint current_partner_id;
      uint id;
      uint key;
      string name;
      string platform_name;
      string model_name;
      string guid;
      string device_ip;
      string device_rapi_ip;
      uint device_rapi_port;
      string password;
      bool password_protected;
      bool password_provided;

      public Device ()
      {
         path = "";
         os_major = 0;
         os_minor = 0;
         version = 0;
         cpu_type = 0;
         current_partner_id = 0;
         id = 0;
         name = "";
         platform_name = "";
         model_name = "";
         guid = "";
         device_ip = "";
         device_rapi_ip = "";
         device_rapi_port = 0;
         password = "";
         key = 0;
         password_protected = false;
         password_provided = false;
      }

      public static IDevice FindInstance (string path)
      {
         IDevice device = null;
         
         if (!Bus.System.NameHasOwner ("org.wimo.dccm"))
            throw new ApplicationException ("Could not find org.wimo.dccm");
      
         device = Bus.System.GetObject<IDevice>("org.wimo.dccm",
                           new ObjectPath (path));
         
         if (device == null)
            throw new ApplicationException (string.Format (
               "The {0} object could not be located " +
               "on the DBUs interface org.wimo.dccm", path));
            
         return device;
      }
      
      public bool PasswordProtected {
         get {
            return password_protected;
         }
         set {
            password_protected = value;
         }
      }

      public bool PasswordProvided {
         get {
            return password_provided;
         }
         set {
            password_provided = value;
         }
      }
      
      public string Password {
         get {
            return password;
         }
         set {
            password = value;
            password_provided = true;
         }
      }
      
      public uint Key {
         get {
            return key;
         }
         set {
            key = value;
         }
      }
      
      public string Path {
         get {
            return path;
         }
         set {
            path = value;
         }
      }
      
      public uint OsVersionMajor {
         get {
            return os_major;
         }
         set {
            os_major = value;
         }
      }
      
      public uint OsVersionMinor {
         get {
            return os_minor;
         }
         set {
            os_minor = value;
         }
      }

      public uint Version {
         get {
            return version;
         }
         set {
            version = value;
         }
      }
      
      public string DeviceIP {
         get {
            return device_ip;
         }
         set {
            device_ip = value;
         }
      }
      
      public string DeviceRapiIP {
         get {
            return device_rapi_ip;
         }
         set {
            device_rapi_ip = value;
         }
      }
      
      public uint DeviceRapiPort {
         get {
            return device_rapi_port;
         }
         set {
            device_rapi_port = value;
         }
      }
      
      public uint CpuType {
         get {
            return cpu_type;
         }
         set {
            cpu_type = value;
         }
      }
      
      public uint CurrentPartnerId {
         get {
            return current_partner_id;
         }
         set {
            current_partner_id = value;
         }
      }
      
      public uint Id {
         get {
            return id;
         }
         set {
            id = value;
         }
      }
      
      public string Name {
         get {
            return name;
         }
         set {
            name = value;
         }
      }
      
      public string PlatformName {
         get {
            return platform_name;
         }
         set {
            platform_name = value;
         }
      }
      
      public string ModelName {
         get {
            return model_name;
         }
         set {
            model_name = value;
         }
      }

      public string Guid {
         get {
            return guid;
         }
         set {
            guid = value;
         }
      }
   }
}
