/*
 * $Id: DccmDeviceManager.cs 86 2007-03-28 13:19:41Z lloydd $
 */
 
using System;
using System.Collections;
using System.Collections.Generic;
using org.freedesktop.DBus;
using NDesk.DBus;

namespace org.wimo.dccm
{
   public delegate void DeviceConnectedEventHandler (string device_path);
   public delegate void DeviceDisconnectedEventHandler (string device_path);
   public delegate void DevicePasswordRequestedEventHandler (string device_path);
   public delegate void DeviceWrongPasswordEventHandler (string device_path);
   public delegate void DeviceConnectionEstablishedEventHandler (string device_path);
   
   [Interface ("org.wimo.dccm.DeviceManager")]
   public interface IDeviceManager
   {
      event DeviceConnectedEventHandler DeviceConnected;
      event DeviceDisconnectedEventHandler DeviceDisconnected;
      event DevicePasswordRequestedEventHandler DevicePasswordRequested;
      event DeviceWrongPasswordEventHandler DeviceWrongPassword;
      event DeviceConnectionEstablishedEventHandler DeviceConnectionEstablished;
      
      string[] GetAllDevices ();
   }
   
   public class DeviceManager : IDeviceManager
   {
      List<Device> devices;

      public event DeviceConnectedEventHandler DeviceConnected;
      public event DeviceDisconnectedEventHandler DeviceDisconnected;
      public event DevicePasswordRequestedEventHandler DevicePasswordRequested;
      public event DeviceWrongPasswordEventHandler DeviceWrongPassword;
      public event DeviceConnectionEstablishedEventHandler DeviceConnectionEstablished;
      
      public DeviceManager ()
      {
         devices = new List<Device> ();
      }
      
      public static IDeviceManager FindInstance ()
      {
        return DeviceManager.FindInstance ("/org/wimo/dccm/DeviceManager");
      }
      
      public static IDeviceManager FindInstance (string path)
      {
         IDeviceManager manager = null;
         
         if (!Bus.System.NameHasOwner ("org.wimo.dccm"))
            throw new ApplicationException ("Could not find org.wimo.dccm");
         
         manager = Bus.System.GetObject<IDeviceManager> ("org.wimo.dccm",
            new ObjectPath (path));

         if (manager == null)
            throw new ApplicationException (string.Format (
               "The {0} object could not be located " +
               "on the DBUs interface org.wimo.dccm", path));
         
         return manager;
      }
      
      public void Register ()
      {
         Bus.System.Register (
            "org.wimo.dccm",
            new ObjectPath ("/org/wimo/dccm/DeviceManager"),
            this);
         
         Bus.System.RequestName ("org.wimo.dccm");
      }
      
      public void StartService ()
      {
         if (!Bus.System.NameHasOwner ("org.wimo.dccm"))
            Bus.System.StartServiceByName ("org.wimo.dccm");
      }
      
      public Device this [string device_path] {
         get {
            foreach (Device dev in devices)
               if (dev.Path == device_path)
                  return dev;
            
            return null;
         }
      }
      
      public string[] GetAllDevices ()
      {
         List<string> paths = new List<string> ();
         
         foreach (Device dev in devices)
            paths.Add (dev.Path);
            
         return paths.ToArray ();
      }
      
      public void Add (Device dev)
      {
         lock (devices) {
            dev.Path = "/org/wimo/dccm/Device/" +
            dev.DeviceIP.Replace ('.', '_') + "_" +
               DateTime.Now.ToFileTime ().ToString ();
            
            devices.Add (dev);
            
            Bus.System.Register (
               "org.wimo.dccm",
               new ObjectPath (dev.Path),
               dev);
         }
      }
      
      public void Remove (Device dev)
      {
         lock (devices) {
            devices.Remove (dev);
   
            Bus.System.Unregister (
               "org.wimo.dccm",
               new ObjectPath (dev.Path));
         }
      }

      public void Connected (Device dev)
      {
         if (DeviceConnected != null)
            DeviceConnected (dev.Path);
      }

      public void Disconnected (Device dev)
      {
         if (DeviceDisconnected != null)
            DeviceDisconnected (dev.Path);
      }
      
      public void PasswordRequested (Device dev)
      {
         if (DevicePasswordRequested != null)
            DevicePasswordRequested (dev.Path);
      }
      
      public void WrongPassword (Device dev)
      {
         if (DeviceWrongPassword != null)
            DeviceWrongPassword (dev.Path);
      }

      public void ConnectionEstablished (Device dev)
      {
         if (DeviceConnectionEstablished != null)
            DeviceConnectionEstablished (dev.Path);
      }
   }
}
