/*
 * $Id: dccm-dbus-client.c 95 2007-03-29 10:02:02Z lloydd $
 */

#include <glib.h>
#include <dbus/dbus-glib.h>

#include "dccm-device-glue.h"
#include "dccm-device-manager-glue.h"

DBusGConnection *bus;

void PrintDevice (const char *device_path)
{
   GError *error = NULL;
   
   DBusGProxy * device_proxy = dbus_g_proxy_new_for_name (bus,
		"org.wimo.dccm",
		device_path,
		"org.wimo.dccm.Device");

   g_print ("-----------\n");
   
   if (!device_proxy) {
      g_warning ("Device proxy %s not found", device_path);
      return;
   }

   char* path = NULL;
   char* ip_address = NULL;
   char* name = NULL;
   char* platform_name = NULL;
   char* model_name = NULL;
   char* guid = NULL;
   char* password = NULL;
   gint os_major = 0;
   gint os_minor = 0;
   gint version = 0;
   gint cpu_type = 0;
   gint cur_partner_id = 0;
   gint id = 0;
   guchar key = 0;
   
   if (!org_wimo_dccm_Device_get_path (device_proxy, &path, &error))
      goto print_error;

   if (!org_wimo_dccm_Device_get_os_version_major (device_proxy, &os_major, &error))
      goto print_error;

   if (!org_wimo_dccm_Device_get_os_version_minor (device_proxy, &os_minor, &error))
      goto print_error;

   if (!org_wimo_dccm_Device_get_version (device_proxy, &version, &error))
      goto print_error;
   
   if (!org_wimo_dccm_Device_get_ip_address (device_proxy, &ip_address, &error))
      goto print_error;
   
   if (!org_wimo_dccm_Device_get_cpu_type (device_proxy, &cpu_type, &error))
      goto print_error;
   
   if (!org_wimo_dccm_Device_get_current_partner_id (device_proxy, &cur_partner_id, &error))
      goto print_error;
   
   if (!org_wimo_dccm_Device_get_id (device_proxy, &id, &error))
      goto print_error;
   
   if (!org_wimo_dccm_Device_get_name (device_proxy, &name, &error))
      goto print_error;
   
   if (!org_wimo_dccm_Device_get_platform_name (device_proxy, &platform_name, &error))
      goto print_error;
   
   if (!org_wimo_dccm_Device_get_model_name (device_proxy, &model_name, &error))
      goto print_error;
   
   if (!org_wimo_dccm_Device_get_guid (device_proxy, &guid, &error))
      goto print_error;
   
   if (!org_wimo_dccm_Device_get_password (device_proxy, &password, &error))
      goto print_error;
   
   if (!org_wimo_dccm_Device_get_key (device_proxy, &key, &error))
      goto print_error;
   
   goto print;

print_error:
   g_warning ("Failed to get attribute: %s", error->message);
   g_error_free (error);

print:
   g_print ("  Path: \"%s\"\n", (path ? path : "null"));
   g_print ("  IP: \"%s\"\n", (ip_address ? ip_address : "null"));
   g_print ("  Name: \"%s\"\n", (name ? name : "null"));
   g_print ("  PlatformName: \"%s\"\n", (platform_name ? platform_name : "null"));
   g_print ("  ModelName: \"%s\"\n", (model_name ? model_name : "null"));
   g_print ("  Guid: \"%s\"\n", (guid ? guid : "null"));
   g_print ("  OsVersionMajor: %d\n", os_major);
   g_print ("  OsVersionMinor: %d\n", os_minor);
   g_print ("  OsVersionBuild: %d\n", version);
   g_print ("  Password: \"%s\"\n", (password ? password : "null"));
   g_print ("  Key: %d\n", key);
   g_print ("  CPUType: %d\n", cpu_type);
   g_print ("  CurrentPartnerId: %d\n", cur_partner_id);
   g_print ("  Id: %d\n", id);

   if (path) g_free (path);
   if (ip_address) g_free (ip_address);
   if (name) g_free (name);
   if (platform_name) g_free (platform_name);
   if (model_name) g_free (model_name);
   if (guid) g_free (guid);
   if (password) g_free (password);
}

int ListAllDevices (DBusGProxy *proxy)
{
   GError* error = NULL;
   char** device_list = 0;
   char** device_list_ptr = 0;

   if (!org_wimo_dccm_DeviceManager_get_all_devices (proxy, &device_list, &error)) {
      g_error ("Failed to get list of connected devices: %s", error->message);
      g_error_free (error);
      return 1;
   }

   g_print ("device_list:\n");
   
   if (device_list) {
      for (device_list_ptr = device_list; *device_list_ptr; device_list_ptr++)
         PrintDevice (*device_list_ptr);
      g_strfreev (device_list);
   } else
      g_print ("  Empty\n");
      
   return 0;
}

static void
device_connected (DBusGProxy *proxy,
		    const char *device_path,
		    gpointer   user_data)
{
   g_print ("connected device %s\n", device_path);
   
   PrintDevice (device_path);
}

static void
device_disconnected (DBusGProxy *proxy,
		    const char *device_path,
		    gpointer   user_data)
{
   g_print ("disconnected device %s\n", device_path);
}

static void
device_password_requested (DBusGProxy *proxy,
		    const char *device_path,
		    gpointer   user_data)
{
   GError *error = NULL;

   g_print ("device %s password requested\n", device_path);
   
   PrintDevice (device_path);

   DBusGProxy * device_proxy = dbus_g_proxy_new_for_name (bus,
		"org.wimo.dccm",
		device_path,
		"org.wimo.dccm.Device");

   g_print ("-----------\n");
   
   if (!device_proxy) {
      g_warning ("Device proxy %s not found", device_path);
      return;
   }

   if (!org_wimo_dccm_Device_provide_password (device_proxy, "1111", &error)) {
      g_error ("Failed to provide device password: %s", error->message);
      g_error_free (error);
      return;
   }
   
   g_print ("Password provided\n");
   g_print ("-----------\n");
}

int main (int argc, char **argv)
{
   GMainLoop *loop;
   DBusGProxy *device_manager_proxy;
   GError* error = NULL;

   g_type_init ();
   
   loop = g_main_loop_new (NULL, FALSE);
   
   bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
   
   if (!bus) {
      g_critical ("Failed to connect to the D-BUS daemon: %s", error->message);
      g_error_free (error);
      return 1;
   }
   
   device_manager_proxy = dbus_g_proxy_new_for_name (bus,
		"org.wimo.dccm",
		"/org/wimo/dccm/DeviceManager",
		"org.wimo.dccm.DeviceManager");

   if (!device_manager_proxy) {
      g_critical ("Service org.wimo.dccm.DeviceManager not found");
      return 1;
   }

   if (ListAllDevices (device_manager_proxy))
      return 1;

   dbus_g_proxy_add_signal (device_manager_proxy,
			   "DeviceConnected",
			   G_TYPE_STRING,
			   G_TYPE_INVALID);

   dbus_g_proxy_add_signal (device_manager_proxy,
			   "DeviceDisconnected",
			   G_TYPE_STRING,
			   G_TYPE_INVALID);
  
   dbus_g_proxy_add_signal (device_manager_proxy,
			   "DevicePasswordRequested",
			   G_TYPE_STRING,
			   G_TYPE_INVALID);
  
   dbus_g_proxy_connect_signal (device_manager_proxy,
      "DeviceConnected",
      G_CALLBACK (device_connected),
      NULL,
      NULL);

   dbus_g_proxy_connect_signal (device_manager_proxy,
      "DeviceDisconnected",
      G_CALLBACK (device_disconnected),
      NULL,
      NULL);
   
   dbus_g_proxy_connect_signal (device_manager_proxy,
      "DevicePasswordRequested",
      G_CALLBACK (device_password_requested),
      NULL,
      NULL);
   
   g_main_loop_run (loop);
  
   g_object_unref (device_manager_proxy);
   
   return 0;
}
